﻿using System;
using System.Xml.Serialization;
using System.IO;
using ICSharpCode.SharpZipLib.Zip.Compression;
using ICSharpCode.SharpZipLib.Zip;

namespace dezentrale.model
{
    public class Attachment
    {
        [XmlAttribute] public string FileName { get; set; } = "";
        [XmlAttribute] public DateTime Added { get; set; } = DateTime.Now;
        [XmlAttribute] public DateTime Modified { get; set; }
        [XmlAttribute] public int Size { get; set; } = 0;
        [XmlAttribute] public string DataFileName { get; set; } = null;
        [XmlIgnore]
        public string CompletePath
        {
            get
            {
                return System.IO.Path.Combine(Program.config.DbDirectory, DataFileName);
            }
        }

        public Attachment() : base() { }
        public Attachment(string fileName) : base()
        {
            Added = DateTime.Now;
            FileName = fileName;
        }
        public Attachment(string fileName, byte[] contents) : base()
        {
            Added = DateTime.Now;
            FileName = fileName;
            SetContents(contents);
        }

        public byte[] GetContents()
        {
            if (DataFileName == null) return null;
            using (ZipInputStream s = new ZipInputStream(File.OpenRead(CompletePath)))
            {
                ZipEntry theEntry;
                while ((theEntry = s.GetNextEntry()) != null)
                {

                    Console.WriteLine(theEntry.Name);
                    string fileName = Path.GetFileName(theEntry.Name);

                    if (fileName == FileName)
                    {
                        long size = theEntry.Size;
                        if (size != this.Size) throw new Exception($"Size in file {DataFileName} ({FileName}) differs from expected.");
                        byte[] contents = new byte[size];
                        size = s.Read(contents, 0, (int)size);
                        if (size != Size) throw new Exception($"Size in file {DataFileName} ({FileName}) differs from expected.");
                        return contents;
                    }
                }
            }

            return null;
        }

        public void SetContents(byte[] contents, bool deleteOldContentFile = true)
        {
            Modified = DateTime.Now;
            if (!string.IsNullOrEmpty(DataFileName))
            {
                //Delete old contents
                try
                {
                    File.Delete(CompletePath);
                    Console.WriteLine($"Old content file {DataFileName} deleted.");
                    DataFileName = null;
                    Size = 0;
                }
                catch (Exception ex)
                {
                    Console.WriteLine($"Cannot delete file {DataFileName}:");
                    Console.WriteLine(ex.Message);
                }
            }
            else
            {
                Console.WriteLine($"Old content file {DataFileName} kept. This is an orphan now.");
            }

            if (contents != null)
            {
                do
                {
                    DataFileName = $"{Guid.NewGuid().ToString()}.zip";
                } while (File.Exists(CompletePath)); //should never happen but we're checking to be sure.

                string completePath = System.IO.Path.Combine(Program.config.DbDirectory, DataFileName);
                Size = contents.Length;

                using (ZipOutputStream s = new ZipOutputStream(File.Create(completePath)))
                {
                    s.SetLevel(9); // 0 - store only ... 9 - best compression

                    var entry = new ZipEntry(FileName) { DateTime = DateTime.Now };
                    s.PutNextEntry(entry);

                    s.Write(contents, 0, contents.Length);
                    s.Finish();
                    s.Close();
                }
            }
        }

        public bool SaveData()
        {
            Program.config.DbChangedSinceExport = true;
            Program.config.LastDbLocalChange = DateTime.Now;
            return XmlData.SaveToFile(Program.ConfigFile, Program.config);
        }
    }
}
